package me.unurled.raxen.manager.server;

import com.onarandombox.MultiverseCore.api.MVWorldManager;
import com.onarandombox.MultiverseCore.api.MultiverseWorld;
import me.unurled.raxen.Raxen;
import org.bukkit.World;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static me.unurled.raxen.utils.Utils.*;

public class WorldManager {

    private Raxen main;

    MVWorldManager worldManager;

    /**
     * String : name
     * MultiverseWorld : a world that need to be registered
     */
    private Map<String, MultiverseWorld> loadedWorlds = new HashMap<>();

    private List<String> worlds = new ArrayList<>();

    private File pluginFolder;
    private File worldFolder;

    public WorldManager(Raxen main) {
        this.main = main;
        this.pluginFolder = new File(main.getDataFolder() + "/worlds/");
        this.worldFolder = new File("worlds/");
        this.worldManager = main.getManager().getMultiverse().getWorldManager();
        worlds.add("Forest");
    }

    /**
     * remove a world
     * @param name
     */
    public void removeWorld(String name) {
        File world = new File(worldFolder + name + "/");
        if (!world.exists() || !world.isDirectory()) {
            error("Saving world " + name + ". Folder don't exists.");
            return;
        }
        worldManager.unloadWorld(name);
        try {
            removeDirectory(world.toPath());
        } catch (IOException e) {
            error("Error while Saving world " + name + " at coping data.");
            e.printStackTrace();
        }
    }

    /**
     * load all world in the worlds folder
     */
    public void load() {
        if (!pluginFolder.exists()) {
            pluginFolder.mkdirs();
            error("Please put in the folder " + pluginFolder.getAbsolutePath() + " all the needed worlds :");
            for (String s : worlds) {
                error(" - " + s + ",");
            }
            return;
        }
        for (String s : worlds) {
            File world = new File(pluginFolder + s);
            File worldd = new File(worldFolder + s);
            if (worldd.exists()) {
                if (worldd.isDirectory()) {
                    worldManager.loadWorld(s);
                }
            } else if (world.exists() ) {
                if (world.isDirectory()) {
                    loadWorld(s);
                } else {
                    error("World " + s + " is not a world Folder!");
                }
            } else {
                error("World " + s + " do not exist.");
            }
        }
    }

    /**
     * load world from the name of it
     * @param name
     */
    public void loadWorld(String name) {
        File world = new File(pluginFolder + "/" + name + "/");
        if (!world.exists() || !world.isDirectory()) {
            error("Loading world " + name + ". Folder " + world.getAbsolutePath() + " don't exists.");
            return;
        }
        File newWorld = new File(worldFolder + "/" + name);
        try {
            log(colorTextComp("<aqua>Copping world " + world.getName() + ".</aqua>"), colorTextComp("<blue>From " + world.getAbsolutePath() + " to " + newWorld.getAbsolutePath() +".</blue>"));
            copyDirectory(world.getAbsolutePath(), newWorld.getAbsolutePath());
        } catch (IOException e) {
            error("Error while Loading world " + name + " at coping data.");
            e.printStackTrace();
        }
        worldManager.loadWorld(name);
        worldManager.addWorld(world.getName(), World.Environment.NORMAL, null, null, false, null);
    }
}
